'###############################################################################
'##                                                                           ##
'##                            U N I M O D U L E                              ##
'##                       ===========================                         ##
'##                                                                           ##
'##  Description: EnOcean Unimodule for temperature measurement               ##
'##                                                                           ##
'##  Version:     1.1                                       Date: March 2013  ##
'##                                                                           ##
'##  Author:      Hartmut Wendt                                               ##
'##  URL:         www.hwhardsoft.de                                           ##
'##                                                                           ##
'##  Copyright:   GNU Public License V3                                       ##
'##                                                                           ##
'##  note:                                                                    ##
'##  This program is free software: you can redistribute it and/or modify     ##
'##  it under the terms of the GNU General Public License as published by     ##
'##  the Free Software Foundation, either version 3 of the License, or        ##
'##  (at your option) any later version.                                      ##
'##  This program is distributed in the hope that it will be useful,          ##
'##  but WITHOUT ANY WARRANTY; without even the implied warranty of           ##
'##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            ##
'##  GNU General Public License for more details.                             ##
'##  You should have received a copy of the GNU General Public License        ##
'##  along with this program.  If not, see <http://www.gnu.org/licenses/>.    ##
'##                                                                           ##
'###############################################################################
$regfile = "m328pdef.dat"
$crystal = 7372800
'$crystal = 8000000
$baud = 9600

$hwstack = 100
$swstack = 48
$framesize = 64

Const False = 0 : Const True = 1
Const Pm_temperature = 0
Const Pm_inputs = 1
Const Pm_menu = 2
Const Sw_ver = "Version 1.1"
Const Sw_date = "build 2013-04-04"


Declare Sub Factory_settings
Declare Sub Show_baseid



'Dim B1 As Byte
'###############################################################################
'##      L C D                                                                ##
'###############################################################################

Ddrb = &B11111110                                           'Port B alles auf Ausgang
Portb = &B11111110                                          'Alle Pullups ran
Rs Alias Portb.1                                            'Verkabelung = Selbsterklrend...
Csb Alias Portb.2
Si Alias Portb.3
Clk Alias Portb.5
Backlight_on Alias Portb.4

$include "inc\dogm.bas"

Set Portd.2
Set Portd.3
Set Portd.4                                                 'Pull-ups ein


'###############################################################################
'##      E E P R O M  V A R I A B L E S                                       ##
'###############################################################################
Dim B_baseid1 As Eram Byte At 0                             'BaseID byte 1
Dim B_baseid2 As Eram Byte At 1                             'BaseID byte 2
Dim B_baseid3 As Eram Byte At 2                             'BaseID byte 3
Dim B_baseid4 As Eram Byte At 3                             'BaseID byte 4

'### Sensor 1...4 ###
Dim B_sensor1_type As Eram Byte At 8
Dim B_sensor2_type As Eram Byte At 9
Dim B_sensor3_type As Eram Byte At 10
Dim B_sensor4_type As Eram Byte At 11

'### Thermostate 1 ###
Dim I_therm1_temp As Eram Integer At &H10
Dim B_therm1_ch As Eram Byte At &H12
Dim B_therm1_underrun As Eram Byte At &H13
Dim B_therm1_overrun As Eram Byte At &H14
Dim B_therm1_invert As Eram Byte At &H15

'### Thermostate 2 ###
Dim I_therm2_temp As Eram Integer At &H20
Dim B_therm2_ch As Eram Byte At &H22
Dim B_therm2_underrun As Eram Byte At &H23
Dim B_therm2_overrun As Eram Byte At &H24
Dim B_therm2_invert As Eram Byte At &H25

'### Thermostate 3 ###
Dim I_therm3_temp As Eram Integer At &H30
Dim B_therm3_ch As Eram Byte At &H32
Dim B_therm3_underrun As Eram Byte At &H33
Dim B_therm3_overrun As Eram Byte At &H34
Dim B_therm3_invert As Eram Byte At &H35

'### Thermostate 4 ###
Dim I_therm4_temp As Eram Integer At &H40
Dim B_therm4_ch As Eram Byte At &H42
Dim B_therm4_underrun As Eram Byte At &H43
Dim B_therm4_overrun As Eram Byte At &H44
Dim B_therm4_invert As Eram Byte At &H45



'###############################################################################
'##      L C D   M E N U                                                      ##
'###############################################################################
Macro Menu_include_data
   ' include the data file created with the menu designer
   $include "menu_data.bas"
End Macro
$include "inc\menu.bas"


'###############################################################################
'##      T E M P E R A T U R E  S E N S O R S                                 ##
'###############################################################################

$include "inc\ds18S20.bas"


'###############################################################################
'##      E N O C E A N  T C M 3 x 0                                           ##
'###############################################################################

Config Com1 = 9600 , Synchrone = 0 , Parity = None , Stopbits = 1 , Databits = 8 , Clockpol = 0

'setup to use a serial output buffer
'and reserve 20 bytes for the buffer
Config Serialout = Buffered , Size = 20
                                        ' use baud rate
$include "inc\tcm3x0.bas"


'###############################################################################
'##      V A R I A B L E S                                                    ##
'###############################################################################
Dim Tempbyte As Byte
Dim Channel As Byte
Dim S3 As String * 16
Dim Pmode As Byte
Dim Tmt_counter As Byte
Dim Input1_state As Bit
Dim Input2_state As Bit
Dim Input3_state As Bit
Dim Input4_state As Bit
Dim Menu_timeout As Integer
Dim B1 As Byte
Dim B2 As Byte
Dim I1 As Integer

'###############################################################################
'##      I N I T                                                              ##
'###############################################################################
Dogm_init                                                   'init LCD driver
Menu_init                                                   'init menu control

Tmt_counter = 1                                             'init transmittion counter
Gosub Draw_homescreen


'###############################################################################
'##      M A I N   L O O P                                                    ##
'###############################################################################
Dogm_clear
Dogm_locate 1 , 5
Dogm_lcd "UniModul"
Dogm_locate 2 , 3
'Dogm_lcd "1.0"
Dogm_lcd Sw_ver
Wait 1
Dogm_clear

'----- request base ID from TCM3x0 --------------------------------------------
Dogm_locate 1 , 1
Dogm_lcd "Request BaseID"
Request_tcm3x0_baseid                                       'request the base id
Show_baseid
Wait 1
Dogm_clear

Do


  '--- read in buttons for menu control ---------------------------------------
   Debounce Pind.3 , 0 , Button_left , Sub
   Debounce Pind.4 , 0 , Button_right , Sub
   Debounce Pind.2 , 0 , Button_enter , Sub


  If Pmode = Pm_temperature Then

  '--- read in binary sensor inputs     ---------------------------------------
  ' high states - contact closed
   Debounce Pind.5 , 1 , Binary4_high , Sub                 'input4
   Debounce Pind.6 , 1 , Binary1_high , Sub                 'input1
   Debounce Pind.7 , 1 , Binary2_high , Sub                 'input2
   Debounce Pinb.0 , 1 , Binary3_high , Sub                 'input3

  'low states - contacts opend
   Debounce Pind.5 , 0 , Binary4_low , Sub
   Debounce Pind.6 , 0 , Binary1_low , Sub
   Debounce Pind.7 , 0 , Binary2_low , Sub
   Debounce Pinb.0 , 0 , Binary3_low , Sub

  '--- increment channel counter ----------------------------------------------
  Channel = Channel + 1
  If Channel > 4 Then Channel = 1

  '--- measure temperature ----------------------------------------------------
  Temp_messen Channel

  '--- display temperature values ---------------------------------------------

   If Channel = 1 Then
      Dogm_locate 1 , 1
   Elseif Channel = 2 Then
      Dogm_locate 2 , 1
   Elseif Channel = 3 Then
      Dogm_locate 1 , 12
   Else
      Dogm_locate 2 , 12
   End If
   S3 = Convert_temp(channel)
   Dogm_lcd S3

   '--- alle 100s einen gltigen Tempraturwert senden -------------------------
   If Tmt_counter = 0 Then Tmt_enocean_temp_protocol Channel , False


   '--- counter for transmittion delay-----------------------------------------
   Incr Tmt_counter
   If Tmt_counter > 20 Then Tmt_counter = 0

   '--- Thermostat function ---------------------------------------------------

   'temperature changed?
   If Old_temp(channel) <> Temperatur(channel) Then
       Gosub Threshold1
       Gosub Threshold2
       Gosub Threshold3
       Gosub Threshold4

   End If
   Old_temp(channel) = Temperatur(channel)


  Else
   'menu timeout
   If Menu_timeout > 0 Then
      Decr Menu_timeout
   Else
      Gosub Draw_homescreen
   End If


   'Menu control
   Menu

   Waitms 100
  End If




Loop


'###############################################################################
'##      S U B R O U T I N E S                                                ##
'###############################################################################


'--- sub routines for menu system ---------------------------------------------
Button_left:
 Menu_backward
 Menu_timeout = 150
 Return


Button_right:
 Menu_forward
 Menu_timeout = 150
 Return


Button_enter:
 Pmode = Pm_menu
 Set Backlight_on
 Menu_timeout = 150
 Tempbyte = Menu_enter()
         Select Case Tempbyte
         Case Menu_exit:                                    ' menu closed
            Gosub Draw_homescreen

         Case 11:
            Tmt_enocean_temp_protocol 1 , True
            If Temperatur(1) <> 200 Then
              Gosub Show_send_screen
            Else
              Gosub Show_failed_screen
            End If

         Case 12:
            Tmt_enocean_temp_protocol 2 , True
            If Temperatur(2) <> 200 Then
              Gosub Show_send_screen
            Else
              Gosub Show_failed_screen
            End If

         Case 13:
            Tmt_enocean_temp_protocol 3 , True
            If Temperatur(3) <> 200 Then
              Gosub Show_send_screen
            Else
              Gosub Show_failed_screen
            End If

         Case 14:
            Tmt_enocean_temp_protocol 4 , True
            If Temperatur(4) <> 200 Then
              Gosub Show_send_screen
            Else
              Gosub Show_failed_screen
            End If


         Case 21:
            Tmt_enocean_switch_protocol 5 , Push_0
            Gosub Show_send_screen

         Case 22:
            Tmt_enocean_switch_protocol 6 , Push_0
            Gosub Show_send_screen

         Case 23:
            Tmt_enocean_switch_protocol 7 , Push_0
            Gosub Show_send_screen

         Case 24:
            Tmt_enocean_switch_protocol 8 , Push_0
            Gosub Show_send_screen

         Case 31:
            'factory settings
            Factory_settings
            Dogm_locate 1 , 1
            Dogm_lcd "factory settings"
            Dogm_locate 2 , 1
            Dogm_lcd "restored!"

         Case 32:
            'version Unimodul
            Dogm_locate 1 , 1
            Dogm_lcd Sw_ver
            Dogm_locate 2 , 1
            Dogm_lcd Sw_date

         Case 33:
            'version TCM320
            Dogm_locate 1 , 1
            Dogm_lcd "TCM320 Version:"
            Request_tcm3x0_sw_ver S3
            Dogm_locate 2 , 1
            Dogm_lcd S3

         Case 34:
            Dogm_locate 1 , 1
            Dogm_lcd "TCM320 BaseID:"
            Show_baseid


         End Select
 Return

Draw_homescreen:
   Dogm_clear
   Pmode = Pm_temperature
   Menu_timeout = 0
   Menu_current_entry = 0
   Reset Backlight_on
Return


Show_send_screen:
   Dogm_locate 1 , 2
   Dogm_lcd "learn protocol"
   Dogm_locate 2 , 3
   Dogm_lcd "transmitted!"
Return

Show_failed_screen:
   Dogm_locate 1 , 1
   Dogm_lcd "Please connect"
   Dogm_locate 2 , 1
   Dogm_lcd "a sensor first!"
Return



Sub Factory_settings
'set all eeprom values to factory settings
B_sensor1_type = 0
B_sensor2_type = 0
B_sensor3_type = 0
B_sensor4_type = 0
'thermostat 1
I_therm1_temp = 0
B_therm1_ch = 1
B_therm1_underrun = 0
B_therm1_overrun = 0
B_therm1_invert = 0
'thermostat 2
I_therm2_temp = 0
B_therm2_ch = 2
B_therm2_underrun = 0
B_therm2_overrun = 0
B_therm2_invert = 0
'thermostat 3
I_therm3_temp = 0
B_therm3_ch = 3
B_therm3_underrun = 0
B_therm3_overrun = 0
B_therm3_invert = 0
'thermostat 4
I_therm4_temp = 0
B_therm4_ch = 4
B_therm4_underrun = 0
B_therm4_overrun = 0
B_therm4_invert = 0

End Sub


Sub Show_baseid
   B1 = B_baseid1                                           'read BaseID Byte 1
   S3 = Hex(b1)
   B1 = B_baseid2                                           'read BaseID Byte 2
   S3 = S3 + " " + Hex(b1)
   B1 = B_baseid3                                           'read BaseID Byte 3
   S3 = S3 + " " + Hex(b1)
   B1 = B_baseid4                                           'read BaseID Byte 4
   S3 = S3 + " " + Hex(b1)

   Dogm_locate 2 , 1
   Dogm_lcd S3                                              'show Base ID

End Sub




'--- sub routines for binary inputs -------------------------------------------
Binary1_high:
 Dogm_locate 2 , 7
 Dogm_lcd "H"
 If Input1_state = 0 Then Tmt_enocean_switch_protocol 9 , Push_1
 Input1_state = 1
 Return

Binary2_high:
 Dogm_locate 2 , 8
 Dogm_lcd "H"
 If Input2_state = 0 Then Tmt_enocean_switch_protocol 10 , Push_1
 Input2_state = 1
 Return

Binary3_high:
 Dogm_locate 2 , 9
 Dogm_lcd "H"
 If Input3_state = 0 Then Tmt_enocean_switch_protocol 11 , Push_1
 Input3_state = 1
 Return

Binary4_high:
 Dogm_locate 2 , 10
 Dogm_lcd "H"
 If Input4_state = 0 Then Tmt_enocean_switch_protocol 12 , Push_1
 Input4_state = 1
 Return

Binary1_low:
 Dogm_locate 2 , 7
 Dogm_lcd "L"
 If Input1_state = 1 Then Tmt_enocean_switch_protocol 9 , Push_0
 Input1_state = 0
 Return

Binary2_low:
 Dogm_locate 2 , 8
 Dogm_lcd "L"
 If Input2_state = 1 Then Tmt_enocean_switch_protocol 10 , Push_0
 Input2_state = 0
 Return

Binary3_low:
 Dogm_locate 2 , 9
 Dogm_lcd "L"
 If Input3_state = 1 Then Tmt_enocean_switch_protocol 11 , Push_0
 Input3_state = 0
 Return

Binary4_low:
 Dogm_locate 2 , 10
 Dogm_lcd "L"
 If Input4_state = 1 Then Tmt_enocean_switch_protocol 12 , Push_0
 Input4_state = 0
 Return


'--- sub routines for thermostat ---------------------------------------------

Threshold1:
'check channel
B1 = B_therm1_ch
If B1 <> Channel Then Return

I1 = I_therm1_temp
'Underrun?
If Temperatur(channel) < I1 Then
   If Old_temp(channel) >= I1 Then
      B1 = B_therm1_underrun
      If B1 <> 0 Then
         'inverse?
         B2 = B_therm1_invert
         If B2 <> 0 Then
            Tmt_enocean_switch_protocol 5 , Push_1

         Else
            Tmt_enocean_switch_protocol 5 , Push_0

         End If
      End If
   End If
'Overrun?
Elseif Temperatur(channel) > I1 Then
   If Old_temp(channel) <= I1 Then
      B1 = B_therm1_overrun
      If B1 <> 0 Then
         'inverse?
         B2 = B_therm1_invert
         If B2 <> 0 Then
            Tmt_enocean_switch_protocol 5 , Push_0

         Else
            Tmt_enocean_switch_protocol 5 , Push_1

         End If
      End If
   End If
End If

Return


Threshold2:
'check channel
B1 = B_therm2_ch
If B1 <> Channel Then Return

I1 = I_therm2_temp
'Underrun?
If Temperatur(channel) < I1 Then
   If Old_temp(channel) >= I1 Then
      B1 = B_therm2_underrun
      If B1 <> 0 Then
         'inverse?
         B2 = B_therm2_invert
         If B2 <> 0 Then
            Tmt_enocean_switch_protocol 6 , Push_1

         Else
            Tmt_enocean_switch_protocol 6 , Push_0

         End If
      End If
   End If
'Overrun?
Elseif Temperatur(channel) > I1 Then
   If Old_temp(channel) <= I1 Then
      B1 = B_therm2_overrun
      If B1 <> 0 Then
         'inverse?
         B2 = B_therm2_invert
         If B2 <> 0 Then
            Tmt_enocean_switch_protocol 6 , Push_0

         Else
            Tmt_enocean_switch_protocol 6 , Push_1

         End If
      End If
   End If
End If

Return


Threshold3:
'check channel
B1 = B_therm3_ch
If B1 <> Channel Then Return

I1 = I_therm3_temp
'Underrun?
If Temperatur(channel) < I1 Then
   If Old_temp(channel) >= I1 Then
      B1 = B_therm3_underrun
      If B1 <> 0 Then
         'inverse?
         B2 = B_therm3_invert
         If B2 <> 0 Then
            Tmt_enocean_switch_protocol 7 , Push_1

         Else
            Tmt_enocean_switch_protocol 7 , Push_0

         End If
      End If
   End If
'Overrun?
Elseif Temperatur(channel) > I1 Then
   If Old_temp(channel) <= I1 Then
      B1 = B_therm3_overrun
      If B1 <> 0 Then
         'inverse?
         B2 = B_therm3_invert
         If B2 <> 0 Then
            Tmt_enocean_switch_protocol 7 , Push_0

         Else
            Tmt_enocean_switch_protocol 7 , Push_1

         End If
      End If
   End If
End If

Return


Threshold4:
'check channel
B1 = B_therm4_ch
If B1 <> Channel Then Return

I1 = I_therm4_temp
'Underrun?
If Temperatur(channel) < I1 Then
   If Old_temp(channel) >= I1 Then
      B1 = B_therm4_underrun
      If B1 <> 0 Then
         'inverse?
         B2 = B_therm4_invert
         If B2 <> 0 Then
            Tmt_enocean_switch_protocol 8 , Push_1

         Else
            Tmt_enocean_switch_protocol 8 , Push_0

         End If
      End If
   End If
'Overrun?
Elseif Temperatur(channel) > I1 Then
   If Old_temp(channel) <= I1 Then
      B1 = B_therm4_overrun
      If B1 <> 0 Then
         'inverse?
         B2 = B_therm4_invert
         If B2 <> 0 Then
            Tmt_enocean_switch_protocol 8 , Push_0

         Else
            Tmt_enocean_switch_protocol 8 , Push_1

         End If
      End If
   End If
End If

Return
