$nocompile
'###############################################################################
'##                                                                           ##
'##                               T C M 3 x 0                                 ##
'##                       ===========================                         ##
'##                                                                           ##
'##  Description: enocean protocol communication according enocean serial     ##
'##               protocol (ESP2)                                             ##
'##                                                                           ##
'##  Version:     1.0                                       Date: March 2013  ##
'##                                                                           ##
'##  Author:      Hartmut Wendt                                               ##
'##  URL:         www.hwhardsoft.de                                           ##
'##                                                                           ##
'##  Copyright:   GNU Public License V3                                       ##
'##                                                                           ##
'##  note:                                                                    ##
'##  This program is free software: you can redistribute it and/or modify     ##
'##  it under the terms of the GNU General Public License as published by     ##
'##  the Free Software Foundation, either version 3 of the License, or        ##
'##  (at your option) any later version.                                      ##
'##  This program is distributed in the hope that it will be useful,          ##
'##  but WITHOUT ANY WARRANTY; without even the implied warranty of           ##
'##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            ##
'##  GNU General Public License for more details.                             ##
'##  You should have received a copy of the GNU General Public License        ##
'##  along with this program.  If not, see <http://www.gnu.org/licenses/>.    ##
'##                                                                           ##
'###############################################################################

'***** PTM200 ****
Const Push_0 = &H70
Const Push_1 = &H50
Const Push_no = &H00

Const Pressed = &H30
Const Released = &H20


Declare Sub Tmt_enocean_trt
Declare Sub Tmt_enocean_switch_protocol(byval Bchannel As Byte , Byval Bdata As Byte)
Declare Sub Tmt_enocean_temp_protocol(byval Bchannel As Byte , Byval Learn As Byte)
Declare Sub Request_tcm3x0_baseid
Declare Sub Request_tcm3x0_sw_ver(byref Sversion As String * 16)

Dim B_enocean(11) As Byte
Dim Inbuffer(14) As Byte


Goto Tcm3x0_jumpover


'###############################################################################
'##      S U B R O U T I N E S                                                ##
'###############################################################################

Sub Tmt_enocean_switch_protocol(byval Bchannel As Byte , Byval Bdata As Byte)
'send switch protocol as "RPS" enocean protocol


   'generate protocol - pressed
   B_enocean(1) = &H6B
   B_enocean(2) = &H05                                      'ORG
   B_enocean(3) = Bdata                                     'Data 3
   B_enocean(4) = 0                                         'Data 2
   B_enocean(5) = 0                                         'Data 1
   B_enocean(6) = 0                                         'Data 0
   B_enocean(7) = B_baseid1
   B_enocean(8) = B_baseid2
   B_enocean(9) = B_baseid3
   B_enocean(10) = B_baseid4
   B_enocean(10) = B_enocean(10) + Bchannel
   B_enocean(11) = Pressed

   'transmit protocol
   Tmt_enocean_trt
   Waitms 50

      'generate protocol - released
   B_enocean(1) = &H6B
   B_enocean(2) = &H05                                      'ORG
   B_enocean(3) = 0                                         'Data 3
   B_enocean(4) = 0                                         'Data 2
   B_enocean(5) = 0                                         'Data 1
   B_enocean(6) = 0                                         'Data 0
   B_enocean(7) = B_baseid1
   B_enocean(8) = B_baseid2
   B_enocean(9) = B_baseid3
   B_enocean(10) = B_baseid4
   B_enocean(10) = B_enocean(10) + Bchannel
   B_enocean(11) = Released

   'transmit protocol
   Tmt_enocean_trt
   Waitms 30

End Sub




Sub Tmt_enocean_temp_protocol(byval Bchannel As Byte , Byval Learn As Byte)
   'send temperature value as "4BS" enocean protocol
   Local Itemp As Integer
   Local Btype As Byte

   Itemp = Temperatur(bchannel)
   If Itemp = 200 Then Exit Sub

   'load sensor type
   If Bchannel = 1 Then
         Btype = B_sensor1_type
   Elseif Bchannel = 2 Then
         Btype = B_sensor2_type
   Elseif Bchannel = 3 Then
         Btype = B_sensor3_type
   Else
         Btype = B_sensor4_type
   End If

   'convert temperature
   If Btype = 0 Then
      'sensor type 0 = -40...+40C
       If Itemp < -40 Then Itemp = -40
       If Itemp > 40 Then Itemp = 40
       Itemp = 40 - Itemp

   Elseif Btype = 1 Then
      'sensor type 1 = +10...+90C
       If Itemp < 10 Then Itemp = 10
       If Itemp > 90 Then Itemp = 90
       Itemp = 90 - Itemp
   Else
      'sensor type 2 = +40...+120C
       If Itemp < 40 Then Itemp = 40
       If Itemp > 120 Then Itemp = 120
       Itemp = 120 - Itemp

   End If

   ' convert to values from 200 ... 0
   Itemp = 255 * Itemp
   Itemp = Itemp / 80

   'generate protocol
   B_enocean(1) = &H6B
   B_enocean(2) = &H07                                      'ORG
   B_enocean(3) = 0                                         'Data 3
   B_enocean(4) = 0                                         'Data 2
   B_enocean(5) = Itemp                                     'Data 1
   If Learn = 0 Then
     B_enocean(6) = 8                                       'Data 0 - Data
   Else
     B_enocean(6) = 0                                       'Data 0 - Learn
   End If
   B_enocean(7) = B_baseid1
   B_enocean(8) = B_baseid2
   B_enocean(9) = B_baseid3
   B_enocean(10) = B_baseid4
   B_enocean(10) = B_enocean(10) + Bchannel
   B_enocean(11) = &H00

   'transmit protocol
   Tmt_enocean_trt
   Waitms 30
End Sub


Sub Request_tcm3x0_baseid
'request the BaseID from TCM3x0 module

Request_id_again:
   'generate RD_BASEID protocol
   B_enocean(1) = &HAB
   B_enocean(2) = &H58                                      'ORG
   B_enocean(3) = 0                                         'Data 3
   B_enocean(4) = 0                                         'Data 2
   B_enocean(5) = 0                                         'Data 1
   B_enocean(6) = 0                                         'Data 0
   B_enocean(7) = 0
   B_enocean(8) = 0
   B_enocean(9) = 0
   B_enocean(10) = &H00
   B_enocean(11) = &H00

   'transmit protocol
   Tmt_enocean_trt

   'read 11 bytes vom TCM3x0 module
   Inputbin Inbuffer(1) , 14

   If Inbuffer(4) = &H98 Then
      B_baseid1 = Inbuffer(5)
      B_baseid2 = Inbuffer(6)
      B_baseid3 = Inbuffer(7)
      B_baseid4 = Inbuffer(8)
      Waitms 100

   Else
      Waitms 100
      Goto Request_id_again

   End If


End Sub


Sub Request_tcm3x0_sw_ver(byref Sversion As String * 16)
'request the software version from TCM3x0 module

Request_ver_again:
   'generate RD_SW_VER protocol
   B_enocean(1) = &HAB
   B_enocean(2) = &H4B                                      'ORG
   B_enocean(3) = 0                                         'Data 3
   B_enocean(4) = 0                                         'Data 2
   B_enocean(5) = 0                                         'Data 1
   B_enocean(6) = 0                                         'Data 0
   B_enocean(7) = 0
   B_enocean(8) = 0
   B_enocean(9) = 0
   B_enocean(10) = &H00
   B_enocean(11) = &H00

   'transmit protocol
   Tmt_enocean_trt

   Do
      Inputbin Inbuffer(1) , 1
   Loop Until Inbuffer(1) = &H5A

   'read 11 bytes vom TCM3x0 module
   Inputbin Inbuffer(1) , 12

   If Inbuffer(2) = &H8C Then
     Sversion = Str(inbuffer(3)) + "." + Str(inbuffer(4)) + "." + _
                Str(inbuffer(5)) + "." + Str(inbuffer(6))

   Else
      'Waitms 100
      'Goto Request_ver_again
      Sversion = "ERR " + Hex(inbuffer(2))

   End If


End Sub



Sub Tmt_enocean_trt
Disable Interrupts
'Transmit enocean radio telegram
  Dim B_calc_checksum As Byte
  Dim B_rx_i As Byte

  'calculate CRC checksum
  B_calc_checksum = 0
  For B_rx_i = 1 To 11
   B_calc_checksum = B_calc_checksum + B_enocean(b_rx_i)
  Next B_rx_i
  'B_enocean(14) = B_calc_checksum
  'transmit sync bytes
  Printbin &HA5
  Printbin &H5A

  'transmit protocol data
  Printbin B_enocean(1) ; 11


  'transmit checksum
  Printbin B_calc_checksum
  Enable Interrupts
End Sub



Tcm3x0_jumpover: